<?php

namespace App\Service;

use App\Exception\ParamsException as ParamsException;
use App\Exception\SqlException as SqlException;

class PaymentService {

    private $paymentFactory;
    private $util;
    
    public function __construct (\App\Factory\PaymentFactory $paymentFactory,
                                 \App\Util\Util $util) {
                                     
        $this->paymentFactory = $paymentFactory;
        $this->util           = $util;
    }

    public function getPaymentCode() {
        $paymentCode = $this->paymentFactory->getPaymentCode();

        if(empty($paymentCode)) 
            throw new ParamsException("Tipo de recebimento padrão não cadastrado.");

        return $paymentCode['CDTIPORECE'];
    }

    public function getMoneyReceiptCode() {
        $paymentCode = $this->paymentFactory->getMoneyReceiptCode();

        if(empty($paymentCode)) 
            throw new ParamsException("Não foi possivel encontrar o tipo de recebimento para pagamento em dinheiro.");

        return $paymentCode;
    }

    public function saveMovcaixadlv($deliveryParams, $orderDetails, $cdfilial, $nrvendarest, $orderPrice, $paymentCode, $cdcliente, $cdconsumidor) {
        $change = 0;
        foreach ($orderDetails['payments'] as $payment) {
            if(($payment["code"] == "money") && (isset($payment["changeFor"])) && (!empty($payment["changeFor"]))) {
                $change       = bcsub($payment["changeFor"], $orderDetails["total"], 2);
                $nrseqmovdlv = $this->util->newCode("MOVCAIXADLV$cdfilial$nrvendarest", 10);
                $this->paymentFactory->saveMovcaixadlv("E", $cdfilial, $nrvendarest, $nrseqmovdlv, $paymentCode, $payment["changeFor"], $cdcliente, $cdconsumidor);
            }else{
                $nrseqmovdlv = $this->util->newCode("MOVCAIXADLV$cdfilial$nrvendarest", 10);
                $this->paymentFactory->saveMovcaixadlv("E", $cdfilial, $nrvendarest, $nrseqmovdlv, $paymentCode, $orderDetails["total"], $cdcliente, $cdconsumidor);
            }
        }
        
        if($change > 0) {
            $nrseqmovdlv = $this->util->newCode("MOVCAIXADLV$cdfilial$nrvendarest", 10);
            $this->paymentFactory->saveMovcaixadlv("S", $cdfilial, $nrvendarest, $nrseqmovdlv, $paymentCode, $change, $cdcliente, $cdconsumidor);
        }

    }
    
    public function getReceiptType($cdFilial, $dlvMode, $tpReceInte) {
        return $this->paymentFactory->getReceiptType($cdFilial, $dlvMode, $tpReceInte);
    }
}