<?php
namespace App\Service;

class WebhookService {
    
    private $webhookFactory;
    private $util;

    const UBR_TOKEN_LIFE      = 604800; // one week
    const AUTH_SCOPE_ORDER    = "order";
    const AUTH_SCOPE_ORDER_V2 = "store.orders.read";
            
    public function __construct (\App\Factory\WebhookFactory $webhookFactory,
                                \App\Util\Util $util) {
        $this->webhookFactory = $webhookFactory;
        $this->util = $util;
    }
    
    public function getClientData($token) {
        /*
        if(strlen($token) != 36) {
            throw new \Exception("Invalid token (36 char)");            
        }
        */
        return $this->webhookFactory->getClientData($token);        
    }

    public function getTokensByClient($clientId) {
        
        return $this->webhookFactory->getTokensByClient($clientId);
    }
    
    public function getClientList() {
        $clientList = $this->webhookFactory->getClientList();

        $i = 0;
        $result = [];

        foreach($clientList as $client) {
            $result[$i] = [
                'idCliente'   => $client['IDCLIENTE'],
                'cliente'     => $client['NOME'],
                'url'         => $client['URL'],
                'tipoCliente' => $client['TIPO'],
                'usuarioDb'   => $client['DBNAME'],
                'version'     => $this->getClientVersion($client['URL']),
                'tokens'      => $this->getTokensByClient($client['IDCLIENTE'])
            ];
            $i++;
        }


        return $result;
    }    

    private function getClientVersion($url) {
        return '1.2.3 - rev 6';
    }

    public function addClient($params) {

        if(!isset($params['nomeCliente']) || empty($params['nomeCliente'])) {
            throw new \Exception("Empty nomeCliente");            
        }

        if(!isset($params['urlCliente']) || empty($params['urlCliente'])) {
            throw new \Exception("Empty urlCliente");            
        }
        
        if(!isset($params['tipoCliente']) || empty($params['tipoCliente'])) {
            throw new \Exception("Empty tipoCliente");            
        }

        if(!isset($params['usuarioDb']) || empty($params['usuarioDb'])) {
            throw new \Exception("Empty usuarioDb");            
        }
        $nomeCliente = trim($params['nomeCliente']);
        $urlCliente  = trim($params['urlCliente']);
        $tipoCliente = trim($params['tipoCliente']);
        $usuarioDb  = trim($params['usuarioDb']);
        $this->webhookFactory->addClient($nomeCliente, $urlCliente, $tipoCliente, $usuarioDb);

        return $this->webhookFactory->getClient($nomeCliente, $urlCliente);
    }

    public function editClient($params) {

        if(!isset($params['idCliente']) || empty($params['idCliente'])) {
            throw new \Exception("Empty idCliente");            
        }

        if(!isset($params['nomeCliente']) || empty($params['nomeCliente'])) {
            throw new \Exception("Empty nomeCliente");            
        }

        if(!isset($params['urlCliente']) || empty($params['urlCliente'])) {
            throw new \Exception("Empty urlCliente");            
        }
        
         if(!isset($params['tipoCliente']) || empty($params['tipoCliente'])) {
            throw new \Exception("Empty tipoCliente");            
        }
        
        if(!isset($params['usuarioDb']) || empty($params['usuarioDb'])) {
            throw new \Exception("Empty usuarioDb");            
        }
        
        $idCliente   = trim($params['idCliente']);
        $nomeCliente = trim($params['nomeCliente']);
        $urlCliente  = trim($params['urlCliente']);
        $tipoCliente = trim($params['tipoCliente']);
        $usuarioDb   = trim($params['usuarioDb']);

        $this->webhookFactory->editClient($idCliente, $nomeCliente, $urlCliente, $tipoCliente, $usuarioDb);

        return $this->webhookFactory->getClient($nomeCliente, $urlCliente);
    }

    public function deleteClient($params) {
        if(!isset($params['idCliente']) || empty($params['idCliente'])) {
            throw new \Exception("Empty idCliente");            
        }

        $idCliente = trim($params['idCliente']);

        if(empty($this->webhookFactory->getClientById($idCliente))) {
            throw new \Exception("Invalid idCliente");            
        }

        $this->webhookFactory->deleteClient($idCliente);
    }

    public function addToken($params) {

        if(!isset($params['idCliente']) || empty($params['idCliente'])) {
            throw new \Exception("Empty idCliente");            
        }

        if(!isset($params['token']) || empty($params['token'])) {
            throw new \Exception("Empty urlCliente");            
        }

        $idCliente = trim($params['idCliente']);
        $token     = trim($params['token']);
        
        if(empty($this->webhookFactory->getClientById($idCliente))) {
            throw new \Exception("Invalid idCliente");            
        }
        
        //validate token
        if(!empty($this->webhookFactory->getTokenByToken($token))) {
            throw new \Exception("O token informado já está associado com algum cliente");            
        }

        $this->webhookFactory->addToken($idCliente, $token);
        return $this->webhookFactory->getToken($idCliente, $token);
    }

    public function editToken($params) {

        if(!isset($params['idToken']) || empty($params['idToken'])) {
            throw new \Exception("Empty idToken");            
        }

        if(!isset($params['token']) || empty($params['token'])) {
            throw new \Exception("Empty token");            
        }

        $idToken = trim($params['idToken']);
        $token   = trim($params['token']);
        
        if(empty($this->webhookFactory->getTokenById($idToken))) {
            throw new \Exception("Invalid idToken");            
        }

        $this->webhookFactory->editToken($idToken, $token);
        return $this->webhookFactory->getTokenById($idToken);
    }

    public function deleteToken($params) {
        if(!isset($params['idToken']) || empty($params['idToken'])) {
            throw new \Exception("Empty idToken");            
        }

        $idToken = trim($params['idToken']);

        if(empty($this->webhookFactory->getTokenById($idToken))) {
            throw new \Exception("Invalid idToken");            
        }

        $this->webhookFactory->deleteToken($idToken);
    }
    
    public function cleanToken() {
        $this->webhookFactory->cleanToken();
    }
    
    public function getAuthTokens() {
        $tokenV1 = '';
        $tokenV2 = '';
        
        $webhookParams = $this->webhookFactory->getWebhookParams();
        
        $clientId      = $webhookParams['UBRCLIENTID'];
        $clientSercret = $webhookParams['UBRCLIENTSECRET'];
        
        if(empty($clientId) || empty($clientSercret))
            throw new \Exception("Invalid clientId or clientSecret");
        
        if($this->isAuthTokenExpired($webhookParams)) {
            
            $tokenV1 = json_decode($this->util->authenticate(self::AUTH_SCOPE_ORDER, $clientId, $clientSercret), true);
            $tokenV2 = json_decode($this->util->authenticate(self::AUTH_SCOPE_ORDER_V2, $clientId, $clientSercret), true);
            
            if(empty($tokenV1))
                throw new \Exception("tokenV1 is empty");
            
            if(empty($tokenV2))
                throw new \Exception("tokenV2 is empty");
            
            $createdAt = time();
            
            $this->webhookFactory->updateWebhookParams($tokenV1, $tokenV2, $createdAt);
        } else {
            $tokenV1 = $webhookParams['UBRTOKENV1'];
            $tokenV2 = $webhookParams['UBRTOKENV2'];
        } 
        
        return [
            'tokenV1' => $tokenV1,
            'tokenV2' => $tokenV2
        ];
    }
    
    private function isAuthTokenExpired($webhookParams) {
        if(empty($webhookParams['UBRTOKENV1']) || empty($webhookParams['UBRTOKENV1']))
            return 1;
        
        if(empty($webhookParams['UBRTKNDTCRIAC']) || ((time() - intval($webhookParams['UBRTKNDTCRIAC'])) > self::UBR_TOKEN_LIFE) )
            return 1;
        
        return 0;
    }

}
